@@ -3,7 +3,7 @@
    xsane-batch-scan.c
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2004 Oliver Rauch
+   Copyright (C) 1998-2007 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -81,7 +81,7 @@ desc[] =
 
 static void xsane_batch_scan_get_parameters(Batch_Scan_Parameters *parameters)
 {
- char buf[255];
+ char buf[TEXTBUFSIZE];
  SANE_Int unit;
 
   DBG(DBG_proc, "xsane_batch_scan_get_parameters\n");
@@ -336,7 +336,7 @@ int xsane_batch_scan_load_list_from_file(char *filename)
 static void xsane_batch_scan_load_list(void)
 {
  char filename[PATH_MAX];
- char windowname[256];
+ char windowname[TEXTBUFSIZE];
 
   DBG(DBG_proc, "xsane_batch_scan_load_list\n");
 
@@ -345,11 +345,11 @@ static void xsane_batch_scan_load_list(void)
   sprintf(windowname, "%s %s %s", xsane.prog_name, WINDOW_LOAD_BATCH_LIST, xsane.device_text);
   xsane_back_gtk_make_path(sizeof(filename), filename, "xsane", "batch-lists", 0, "default", ".xbl", XSANE_PATH_LOCAL_SANE);
 
-  if (!xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, FALSE, FALSE, FALSE, FALSE))
+  if (!xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, NULL, XSANE_FILE_CHOOSER_ACTION_OPEN, XSANE_GET_FILENAME_SHOW_NOTHING, XSANE_FILE_FILTER_ALL | XSANE_FILE_FILTER_BATCHLIST, XSANE_FILE_FILTER_BATCHLIST))
   {
     if (xsane_batch_scan_load_list_from_file(filename)) /* error while loading file ? */
     {
-     char buf[256];
+     char buf[TEXTBUFSIZE];
 
       snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, filename, strerror(errno));
       xsane_back_gtk_error(buf, TRUE);
@@ -387,7 +387,7 @@ static void xsane_batch_scan_save_list(void)
  int fd;
  Wire w;
  char filename[PATH_MAX];
- char windowname[256];
+ char windowname[TEXTBUFSIZE];
 
   DBG(DBG_proc, "xsane_batch_scan_save_list\n");
 
@@ -396,7 +396,7 @@ static void xsane_batch_scan_save_list(void)
   sprintf(windowname, "%s %s %s", xsane.prog_name, WINDOW_SAVE_BATCH_LIST, xsane.device_text);
   xsane_back_gtk_make_path(sizeof(filename), filename, "xsane", "batch-lists", 0, "default", ".xbl", XSANE_PATH_LOCAL_SANE);
 
-  if (!xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, FALSE, FALSE, FALSE, FALSE))
+  if (!xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, NULL, XSANE_FILE_CHOOSER_ACTION_SAVE, XSANE_GET_FILENAME_SHOW_NOTHING, XSANE_FILE_FILTER_ALL | XSANE_FILE_FILTER_BATCHLIST, XSANE_FILE_FILTER_BATCHLIST));
   {
     fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0600);
 
@@ -429,7 +429,7 @@ static void xsane_batch_scan_save_list(void)
     }
     else
     {
-     char buf[256];
+     char buf[TEXTBUFSIZE];
 
       snprintf(buf, sizeof(buf), "%s `%s': %s", ERR_OPEN_FAILED, filename, strerror(errno));
       xsane_back_gtk_error(buf, TRUE);
@@ -447,7 +447,7 @@ static void xsane_batch_scan_scan_list(void)
  GList *list = GTK_LIST(xsane.batch_scan_list)->children;
  Batch_Scan_Parameters *parameters = NULL;
  SANE_Int val_start = SANE_TRUE;
- SANE_Int val_loop  = SANE_TRUE;
+ SANE_Int val_loop  = BATCH_MODE_LOOP;
  SANE_Int val_end   = SANE_FALSE;
  SANE_Word val_next_tl_y = SANE_FIX(0.0);
 
@@ -463,7 +463,7 @@ static void xsane_batch_scan_scan_list(void)
   {
     if (!list->next) /* last scan */
     {
-      val_loop = SANE_FALSE;
+      val_loop = BATCH_MODE_LAST_SCAN;
       val_end  = SANE_TRUE;
       val_next_tl_y = SANE_FIX(0.0);
     }
@@ -501,10 +501,11 @@ static void xsane_batch_scan_scan_list(void)
       gtk_main_iteration();
     }
 
-    xsane_scan_dialog();
+    xsane_scan_dialog(NULL);
 
     while (xsane.scanning)
     {
+      /* we MUST call gtk_events_pending() or gdk_input_add will not work! */
       if (gtk_events_pending())
       {
         gtk_main_iteration();
@@ -532,7 +533,7 @@ static void xsane_batch_scan_scan_list(void)
   xsane_control_option(xsane.dev, xsane.well_known.batch_scan_end, SANE_ACTION_SET_VALUE, &val_end, NULL);
   xsane_control_option(xsane.dev, xsane.well_known.batch_scan_next_tl_y, SANE_ACTION_SET_VALUE, &val_next_tl_y, NULL);
 
-  xsane.batch_loop = FALSE; /* make sure we reset the batch scan loop flag */
+  xsane.batch_loop = BATCH_MODE_OFF; /* make sure we reset the batch scan loop flag */
 
   if (parameters)
   {
@@ -560,7 +561,11 @@ static void xsane_batch_scan_scan_selected(void)
       xsane_batch_scan_establish_parameters(parameters, TRUE);
     }
 
-    xsane_scan_dialog();
+    xsane.batch_loop = BATCH_MODE_LAST_SCAN; /* to make sure we do not scan multiple times */
+
+    xsane_scan_dialog(NULL);
+
+    xsane.batch_loop = BATCH_MODE_OFF; /* make sure we reset the batch scan loop flag */
   }
 }
 
@@ -594,7 +599,7 @@ static void xsane_batch_scan_delete(void)
 
 static void xsane_batch_scan_update_label(Batch_Scan_Parameters *parameters)
 {
-  char buf[256];
+  char buf[TEXTBUFSIZE];
   const char *unit_str;
   double tl_x = parameters->tl_x;
   double tl_y = parameters->tl_y;
@@ -842,7 +847,7 @@ static void xsane_batch_scan_rename_callback(GtkWidget *widget, gpointer data)
  GtkWidget *text;
  GtkWidget *button;
  GtkWidget *vbox, *hbox;
- char buf[256];
+ char buf[TEXTBUFSIZE];
 
   DBG(DBG_proc, "xsane_batch_scan_rename\n");
 
@@ -888,6 +893,17 @@ static void xsane_batch_scan_rename_callback(GtkWidget *widget, gpointer data)
       gtk_box_pack_start(GTK_BOX(vbox), text, TRUE, TRUE, 4);
       gtk_widget_show(text);
 
+
+#ifdef HAVE_GTK2
+      button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
+      button = gtk_button_new_with_label(BUTTON_CANCEL);
+#endif
+      g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_batch_scan_rename_button_callback, (void *) -1);
+      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+      gtk_widget_show(button);
+
+
 #ifdef HAVE_GTK2
       button = gtk_button_new_from_stock(GTK_STOCK_OK);
 #else
@@ -900,15 +916,6 @@ static void xsane_batch_scan_rename_callback(GtkWidget *widget, gpointer data)
       gtk_widget_show(button);
 
 
-#ifdef HAVE_GTK2
-      button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-#else
-      button = gtk_button_new_with_label(BUTTON_CANCEL);
-#endif
-      g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_batch_scan_rename_button_callback, (void *) -1);
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-
       xsane_batch_scan_rename = 0;
 
       while (xsane_batch_scan_rename == 0)
@@ -938,6 +945,12 @@ static gint xsane_batch_scan_win_delete(GtkWidget *widget, gpointer data)
 {
   DBG(DBG_proc, "xsane_batch_scan_win_delete\n");
 
+  if (preferences.show_batch_scan)
+  {
+    xsane_window_get_position(xsane.batch_scan_dialog, &xsane.batch_dialog_posx, &xsane.batch_dialog_posy);
+    gtk_window_move(GTK_WINDOW(xsane.batch_scan_dialog), xsane.batch_dialog_posx, xsane.batch_dialog_posy);
+  }
+
   gtk_widget_hide(widget);
   preferences.show_batch_scan = FALSE;
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(xsane.show_batch_scan_widget), preferences.show_batch_scan);
