@@ -3,7 +3,7 @@
    xsane-front-gtk.h
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2004 Oliver Rauch
+   Copyright (C) 1998-2007 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,7 @@
 extern int xsane_parse_options(char *options, char *argv[]);
 extern void xsane_get_bounds(const SANE_Option_Descriptor *opt, double *minp, double *maxp);
 extern double xsane_find_best_resolution(int well_known_option, double dpi);
-extern int xsane_set_resolution(int well_known_option, double resolution);
+extern double xsane_set_resolution(int well_known_option, double resolution);
 extern void xsane_set_all_resolutions(void);
 extern void xsane_define_maximum_output_size(); 
 extern void xsane_close_dialog_callback(GtkWidget *widget, gpointer data);
@@ -57,14 +57,14 @@ extern void xsane_option_menu_new(GtkWidget *parent, char *str_list[], const cha
                                   void *option_menu_callback, SANE_Int settable, const gchar *widget_name);
 extern void xsane_option_menu_new_with_pixmap(GdkWindow *window, GtkBox *parent, const char *xpm_d[], const char *desc,
                                               char *str_list[], const char *val,
-                                              GtkObject **data, int option,
+                                              GtkWidget **data, int option,
                                               void *option_menu_callback, SANE_Int settable, const gchar *widget_name);
 extern void xsane_range_new(GtkBox *parent, char *labeltext, const char *desc,
                             float min, float max, float quant, float page_step,
-                            int digits, double *val, GtkObject **data, void *xsane_range_callback, SANE_Int settable);
+                            int digits, double *val, GtkWidget **data, void *xsane_range_callback, SANE_Int settable);
 extern void xsane_range_new_with_pixmap(GdkWindow *window, GtkBox *parent, const char *xpm_d[], const char *desc,
                                         float min, float max, float quant, float page_step, int digits,
-                                        double *val, GtkObject **data, int option, void *xsane_range_callback, SANE_Int settable);
+                                        double *val, GtkWidget **data, int option, void *xsane_range_callback, SANE_Int settable);
 extern void xsane_set_filename(gchar *filename);
 extern void xsane_outputfilename_new(GtkWidget *vbox);
 extern void xsane_separator_new(GtkWidget *xsane_parent, int dist);
@@ -77,11 +77,15 @@ extern void xsane_define_output_filename(void);
 extern int xsane_identify_output_format(char *filename, char *filetype, char **ext);
 extern void xsane_change_working_directory(void);
 extern int xsane_display_eula(int ask_for_accept);
-extern int xsane_display_gpl(void);
+extern void xsane_display_gpl(void);
 extern void xsane_window_get_position(GtkWidget *gtk_window, gint *x, gint *y);
 extern void xsane_widget_test_uposition(GtkWidget *gtk_window);
 extern int xsane_front_gtk_getname_dialog(const char *dialog_title, const char *desc_text, char *oldname, char **newname);
-extern void xsane_front_gtk_mail_project_update_lockfile_status();
+extern void xsane_front_gtk_list_entries_swap(GtkWidget *list_item_1, GtkWidget *list_item_2);
+extern void xsane_front_gtk_add_process_to_list(pid_t pid);
+extern int xsane_front_gtk_option_defined(char *string);
+extern void xsane_front_gtk_email_project_update_lockfile_status(void);
+extern void xsane_project_dialog_close(void);
 
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
