@@ -3,7 +3,7 @@
    xsane-icons.c
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2005 Oliver Rauch
+   Copyright (C) 1998-2007 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -905,7 +905,7 @@ const char *person_xpm[] =
 
 /* --------------------------------------------------- */
 
-const char *mail_xpm[] =
+const char *email_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
 "    20    20        4            1",
@@ -939,7 +939,7 @@ const char *mail_xpm[] =
 
 /* --------------------------------------------------- */
 
-const char *mailreceiver_xpm[] =
+const char *emailreceiver_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
 "    20    20        3            1",
@@ -1005,6 +1005,110 @@ const char *subject_xpm[] =
 
 /* --------------------------------------------------- */
 
+const char *adf_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"* c #000000",
+". c #ffffff",
+"- c #e0e0e0",
+"  none",
+/* pixels */
+"        ************",
+"        *..........*",
+"      ***..........*",
+"      *.*..........*",
+"    ***.*..........*",
+"    *.*.*..........*",
+"  ***.*.*..........*",
+"  *.*.*.*..........*",
+"  *.*.*.*..........*",
+"  *.*.*.*..........*",
+"  *.*.*.*..........*",
+"  *.*.*.*..........*",
+"  *.*.*.*..........*",
+"  *.*.*.************",
+"  *.*.*..........*  ",
+"  *.*.************  ",
+"  *.*..........*    ",
+"  *.************    ",
+"  *..........*      ",
+"  ************      "
+};
+
+/* --------------------------------------------------- */
+
+
+const char *target_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"+ c #101010",
+"* c #d06060",
+". c #ffffff",
+"  none",
+/* pixels */
+"         **         ",
+"       ++**++       ",
+"     ++..**..++     ",
+"    +....**....+    ",
+"   +...++**++...+   ",
+"  +...+..**..+...+  ",
+"  +..+...**...+..+  ",
+" +..+...+**+...+..+ ",
+" +..+..+.**.+..+..+ ",
+"*********..*********",
+"*********..*********",
+" +..+..+.**.+..+..+ ",
+" +..+...+**+...+..+ ",
+"  +..+...**...+..+  ",
+"  +...+..**..+...+  ",
+"   +...++**++...+   ",
+"    +....**....+    ",
+"     ++..**..++     ",
+"       ++**++       ",
+"         **         "
+};
+
+/* --------------------------------------------------- */
+
+const char *multipage_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        5            1",
+/* colors */
+"* c #000000",
+". c #ffffff",
+"- c #e0e0e0",
+"# c #90a0b0",
+"  none",
+/* pixels */
+"--------------------",
+"-######************-",
+"-######*..........*-",
+"-###****..........*-",
+"-###*..*..........*-",
+"-****..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..*..........*-",
+"-*..*..************-",
+"-*..*..........*###-",
+"-*..************###-",
+"-*..........*######-",
+"-************######-",
+"--------------------"
+};
+
+/* --------------------------------------------------- */
+
 const char *colormode_xpm[] = {
 /* width height num_colors chars_per_pixel */
 "    20    20        23            1",
@@ -1057,6 +1161,38 @@ const char *colormode_xpm[] = {
 
 /* --------------------------------------------------- */
 
+const char *step_xpm[] = {
+/* width height num_colors chars_per_pixel */
+"    20    20        3            1",
+/* colors */
+". c #000000",
+"# c #a0a0a0",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"  ......            ",
+"  ......            ",
+"  .....#.           ",
+"  ......#.          ",
+" ........#.         ",
+" .........#.......  ",
+" .................. ",
+" .................. ",
+" .................. ",
+" ......    .......  ",
+"                    ",
+"                    ",
+"                    "
+};
+
+/* --------------------------------------------------- */
+
 const char *medium_xpm[] = {
 /* width height num_colors chars_per_pixel */
 "    20    20        3            1",
@@ -2774,6 +2910,42 @@ const char *resolution_y_xpm[] =
 
 /* --------------------------------------------------- */
 
+const char *cms_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        6            1",
+/* colors */
+". c #000000",
+"  none",
+"w c #ffffff",
+"r c #ff0000",
+"g c #00ff00",
+"b c #0000ff",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" rrrr  g    g  bbbb ",
+"rrrrrr gg  gg bbbbbb",
+"rr   r gggggg bb   b",
+"rr     gggggg bb    ",
+"rr     gg  gg bbbbb ",
+"rr     gg  gg  bbbb ",
+"rr     gg  gg     bb",
+"rr   r gg  gg b   bb",
+"rrrrrr gg  gg bbbbbb",
+" rrrr  gg  gg  bbbb ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
 const char *scanner_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
