@@ -3,7 +3,7 @@
    xsane-icons.h
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2004 Oliver Rauch
+   Copyright (C) 1998-2007 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -60,10 +60,14 @@ extern const char *mirror_x_xpm[];
 extern const char *mirror_y_xpm[];
 extern const char *fax_xpm[];
 extern const char *faxreceiver_xpm[];
-extern const char *mail_xpm[];
-extern const char *mailreceiver_xpm[];
+extern const char *email_xpm[];
+extern const char *emailreceiver_xpm[];
 extern const char *subject_xpm[];
+extern const char *adf_xpm[];
+extern const char *target_xpm[];
+extern const char *multipage_xpm[];
 extern const char *colormode_xpm[];
+extern const char *step_xpm[];
 extern const char *medium_xpm[];
 extern const char *medium_edit_xpm[];
 extern const char *medium_delete_xpm[];
@@ -115,6 +119,7 @@ extern const char *zoom_y_xpm[];
 extern const char *resolution_xpm[];
 extern const char *resolution_x_xpm[];
 extern const char *resolution_y_xpm[];
+extern const char *cms_xpm[];
 extern const char *scanner_xpm[];
 extern const char *intensity_xpm[];
 extern const char *red_xpm[];
