@@ -3,7 +3,7 @@
    xsane-preview.h
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2005 Oliver Rauch
+   Copyright (C) 1998-2007 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -122,7 +122,7 @@ typedef struct
   char last_offset_16_byte;
   int scan_incomplete;
   int invalid;
-  int preview_colors;
+  int preview_channels;
   time_t image_last_time_updated;
   gint input_tag;
   SANE_Parameters params;
@@ -153,6 +153,13 @@ typedef struct
 
   int show_selection;
 
+#ifdef HAVE_LIBLCMS
+  int cms_enable;
+  int cms_proofing;
+  int cms_proofing_intent;
+  int cms_gamut_check;
+#endif
+
 #if 0
   Batch_selection *batch_selection;
 #endif
