@@ -3,7 +3,7 @@
    xsane.h
 
    Oliver Rauch <Oliver.Rauch@rauch-domain.de>
-   Copyright (C) 1998-2005 Oliver Rauch
+   Copyright (C) 1998-2007 Oliver Rauch
    This file is part of the XSANE package.
 
    This program is free software; you can redistribute it and/or modify
@@ -69,6 +69,12 @@
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_LIBLCMS
+# include "lcms.h"
+#else
+# define cmsHTRANSFORM void *
+#endif
+
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
 #if 0
@@ -85,11 +91,11 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-#define XSANE_VERSION		"0.97"
+#define XSANE_VERSION		"0.994"
 #define XSANE_AUTHOR		"Oliver Rauch"
 #define XSANE_COPYRIGHT		"Oliver Rauch"
-#define XSANE_DATE		"1998-2005"
-#define XSANE_EMAIL		"Oliver.Rauch@xsane.org"
+#define XSANE_DATE		"1998-2007"
+#define XSANE_EMAIL_ADR		"Oliver.Rauch@xsane.org"
 #define XSANE_HOMEPAGE		"http://www.xsane.org"
 #define XSANE_COPYRIGHT_TXT	XSANE_DATE " " XSANE_COPYRIGHT
 
@@ -102,35 +108,48 @@
 #define XSANE_CONTINUOUS_HOLD_TIME	10
 #define XSANE_DEFAULT_DEVICE		"SANE_DEFAULT_DEVICE"
 #define XSANE_3PASS_BUFFER_RGB_SIZE	1024
+#define TEXTBUFSIZE			255
+
+#ifndef M_PI_2
+# define M_PI_2 1.57079632679489661923 /* pi/2 */
+#endif
 
 #ifdef HAVE_WINDOWS_H
-# define BUGGY_GDK_INPUT_EXCEPTION
 # define _WIN32
 #endif
 
+#ifdef _WIN32
+# define BUGGY_GDK_INPUT_EXCEPTION
+#endif
+
+#ifdef HAVE_OS2_H
+# define BUGGY_GDK_INPUT_EXCEPTION
+# define strcasecmp stricmp
+#endif
+
 #ifdef HAVE_LIBPNG
 #ifdef HAVE_LIBZ
-# define XSANE_DEFAULT_MAILTYPE XSANE_FILETYPE_PNG
-# define XSANE_ACTIVATE_MAIL
+# define XSANE_DEFAULT_EMAIL_TYPE XSANE_FILETYPE_PNG
+# define XSANE_ACTIVATE_EMAIL
 #endif
 #endif
 
-#ifndef XSANE_DEFAULT_MAILTYPE
+#ifndef XSANE_DEFAULT_EMAIL_TYPE
 #ifdef HAVE_LIBJPEG
-# define XSANE_DEFAULT_MAILTYPE XSANE_FILETYPE_JPEG
-# define XSANE_ACTIVATE_MAIL
+# define XSANE_DEFAULT_EMAIL_TYPE XSANE_FILETYPE_JPEG
+# define XSANE_ACTIVATE_EMAIL
 #endif
 #endif
 
-#ifndef XSANE_DEFAULT_MAILTYPE
+#ifndef XSANE_DEFAULT_EMAIL_TYPE
 #ifdef HAVE_LIBTIFF
-# define XSANE_DEFAULT_MAILTYPE XSANE_FILETYPE_TIFF
-# define XSANE_ACTIVATE_MAIL
+# define XSANE_DEFAULT_EMAIL_TYPE XSANE_FILETYPE_TIFF
+# define XSANE_ACTIVATE_EMAIL
 #endif
 #endif
 
-#ifndef XSANE_DEFAULT_MAILTYPE
-# define XSANE_DEFAULT_MAILTYPE XSANE_FILETYPE_PNM
+#ifndef XSANE_DEFAULT_EMAIL_TYPE
+# define XSANE_DEFAULT_EMAIL_TYPE XSANE_FILETYPE_PNM
 #endif
 
 
@@ -237,6 +256,11 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+#define IMAGE_SAVED	TRUE
+#define IMAGE_NOT_SAVED	FALSE
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 #include "xsane-text.h"
 #include "xsane-fixedtext.h"
 #include "xsane-icons.h"
@@ -281,12 +305,6 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-#ifdef HAVE_OS2_H
-# define strcasecmp stricmp
-#endif
-
-/* ---------------------------------------------------------------------------------------------------------------------- */
-
 enum
 {
   XSANE_PATH_LOCAL_SANE = 0,
@@ -369,7 +387,7 @@ typedef struct Image_info
     int image_height;
 
     int depth;
-    int colors;
+    int channels;
 
     double resolution_x;
     double resolution_y;
@@ -392,6 +410,13 @@ typedef struct Image_info
     double threshold;
 
     int reduce_to_lineart;
+
+    int enable_color_management;
+    int cms_function;
+    int cms_intent;
+    int cms_bpc;
+
+    char icm_profile[PATH_MAX];
   }
 Image_info;
 
@@ -432,15 +457,43 @@ Image_info;
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-enum { XSANE_VIEWER, XSANE_SAVE, XSANE_COPY, XSANE_FAX, XSANE_MAIL };
-enum { XSANE_LINEART_STANDARD, XSANE_LINEART_XSANE, XSANE_LINEART_GRAYSCALE };
+enum
+{
+  XSANE_VIEWER,
+  XSANE_SAVE,
+  XSANE_COPY,
+  XSANE_MULTIPAGE,
+  XSANE_FAX,
+  XSANE_EMAIL
+};
+
+enum
+{
+  XSANE_LINEART_STANDARD,
+  XSANE_LINEART_XSANE,
+  XSANE_LINEART_GRAYSCALE
+};
+
+enum
+{
+  EMAIL_AUTH_NONE = 0,
+  EMAIL_AUTH_POP3,
+  EMAIL_AUTH_ASMTP_PLAIN,
+  EMAIL_AUTH_ASMTP_LOGIN,
+  EMAIL_AUTH_ASMTP_CRAM_MD5
+};
+
+enum
+{
+  XSANE_CMS_FUNCTION_EMBED_SCANNER_ICM_PROFILE = 0,
+  XSANE_CMS_FUNCTION_CONVERT_TO_SRGB,
+  XSANE_CMS_FUNCTION_CONVERT_TO_WORKING_CS
+};
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
 extern void xsane_pref_save(void);
 extern void xsane_interface(int argc, char **argv);
-extern void xsane_fax_project_save(void);
-extern void xsane_mail_project_save(void);
 extern void xsane_batch_scan_add(void);
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -464,11 +517,12 @@ extern void xsane_batch_scan_add(void);
 #define FAXPOSTSCRIPTOPT	""
 #define FAXNORMALOPT		"-l"
 #define FAXFINEOPT		"-m"
-#define FAXVIEWER 	 	"gv"
+#define FAXVIEWER		"ghostscript"
 #define FAXCONVERTPSTOPNM  	"gs -dNOPAUSE -dBATCH -q -r204 -sDEVICE=pnm -sOutputFile="
-#define MAILPROJECT 	    	"mailproject"
-#define MAILCOMMAND 	 	"sendmail"
-#define MAILVIEWER 	 	"xv"
+#define EMAILPROJECT 	    	"emailproject"
+#define EMAILCOMMAND 	 	"sendmail"
+#define MULTIPAGEPROJECT    	"multipageproject"
+#define MULTIPAGEFILETYPE    	XSANE_FILETYPE_PDF
 #define OCRCOMMAND 	 	"gocr"
 #define OCRINPUTFILEOPT	 	"-i"
 #define OCROUTPUTFILEOPT	"-o"
@@ -490,26 +544,28 @@ extern void xsane_batch_scan_add(void);
 #define XSANE_GAMMA_MIN		0.3
 #define XSANE_GAMMA_MAX		3.0
 
-#define HIST_WIDTH		256
-#define HIST_HEIGHT		100
-#define XSANE_SHELL_WIDTH	296
-#define XSANE_SHELL_HEIGHT	451
-#define XSANE_SHELL_POS_X	1
-#define XSANE_SHELL_POS_Y	50
-#define XSANE_HISTOGRAM_POS_X	280
-#define XSANE_HISTOGRAM_POS_Y	50
-#define XSANE_GAMMA_POS_X	280
-#define XSANE_GAMMA_POS_Y	420
-#define XSANE_BATCH_POS_X	480
-#define XSANE_BATCH_POS_Y	420
-#define XSANE_STD_OPTIONS_POS_X	1
-#define XSANE_STD_OPTIONS_POS_Y	400
-#define XSANE_ADV_OPTIONS_POS_X	280
-#define XSANE_ADV_OPTIONS_POS_Y	420
-#define XSANE_PREVIEW_POS_X	560
-#define XSANE_PREVIEW_POS_Y	50
-#define XSANE_PREVIEW_WIDTH	100
-#define XSANE_PREVIEW_HEIGHT	100
+#define HIST_WIDTH			256
+#define HIST_HEIGHT			100
+#define XSANE_DIALOG_WIDTH		296
+#define XSANE_DIALOG_HEIGHT		451
+#define XSANE_DIALOG_POS_X		1
+#define XSANE_DIALOG_POS_Y		50
+#define XSANE_HISTOGRAM_DIALOG_POS_X	280
+#define XSANE_HISTOGRAM_DIALOG_POS_Y	50
+#define XSANE_PROJECT_DIALOG_POS_X	280
+#define XSANE_PROJECT_DIALOG_POS_Y	425
+#define XSANE_GAMMA_DIALOG_POS_X	280
+#define XSANE_GAMMA_DIALOG_POS_Y	420
+#define XSANE_BATCH_DIALOG_POS_X	480
+#define XSANE_BATCH_DIALOG_POS_Y	420
+#define XSANE_STD_OPTIONS_DIALOG_POS_X	1
+#define XSANE_STD_OPTIONS_DIALOG_POS_Y	400
+#define XSANE_ADV_OPTIONS_DIALOG_POS_X	280
+#define XSANE_ADV_OPTIONS_DIALOG_POS_Y	420
+#define XSANE_PREVIEW_DIALOG_POS_X	560
+#define XSANE_PREVIEW_DIALOG_POS_Y	50
+#define XSANE_PREVIEW_DIALOG_WIDTH	100
+#define XSANE_PREVIEW_DIALOG_HEIGHT	100
 
 #define XSANE_SLIDER_ACTIVE	0
 #define XSANE_SLIDER_INACTIVE	4
@@ -534,7 +590,7 @@ extern void xsane_batch_scan_add(void);
 enum
 {
  XSANE_UNKNOWN, XSANE_PNM, XSANE_JPEG, XSANE_PNG, XSANE_PS, XSANE_TIFF, XSANE_RGBA,
- XSANE_RAW16, XSANE_PNM16, XSANE_TEXT
+ XSANE_RAW16, XSANE_PNM16, XSANE_TEXT, XSANE_PDF
 };
  
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -581,6 +637,7 @@ typedef struct Xsane
     char *backend;
     char *backend_translation;
     char *device_set_filename;
+    char *xsane_rc_set_filename;
     char *output_filename;
     char *dummy_filename;
 
@@ -635,10 +692,10 @@ typedef struct Xsane
 
 
     /* dialogs */
-    GtkWidget *shell;
+    GtkWidget *dialog;
     GtkWidget *menubar;
-    GtkWidget *standard_options_shell;
-    GtkWidget *advanced_options_shell;
+    GtkWidget *standard_options_dialog;
+    GtkWidget *advanced_options_dialog;
     GtkWidget *main_dialog_scrolled;
     GtkWidget *histogram_dialog;
     GtkWidget *gamma_dialog;
@@ -649,28 +706,21 @@ typedef struct Xsane
     GtkWidget *batch_scan_list;
     GtkAdjustment *batch_scan_vadjustment;
 
-    GtkWidget *fax_dialog;
-    GtkWidget *fax_list;
-    GtkWidget *fax_project_box;
-    GtkWidget *fax_project_exists;
-    GtkWidget *fax_project_not_exists;
-    GtkWidget *fax_project_entry;
-    GtkWidget *fax_project_entry_box;
-    GtkWidget *fax_receiver_entry;
-    GtkProgressBar *fax_progress_bar;
+    GtkWidget *project_dialog;
+    GtkWidget *project_list;
+    GtkWidget *project_box;
+    GtkWidget *project_exists;
+    GtkWidget *project_not_exists;
+    GtkWidget *project_entry;
+    GtkWidget *project_entry_box;
+    GtkProgressBar *project_progress_bar;
 
-    GtkWidget *mail_dialog;
-    GtkWidget *mail_list;
-    GtkWidget *mail_project_box;
-    GtkWidget *mail_project_exists;
-    GtkWidget *mail_project_not_exists;
-    GtkWidget *mail_project_entry;
-    GtkWidget *mail_project_entry_box;
-    GtkWidget *mail_receiver_entry;
-    GtkWidget *mail_subject_entry;
-    GtkWidget *mail_text_widget;
-    GtkWidget *mail_html_mode_widget;
-    GtkProgressBar *mail_progress_bar;
+    GtkWidget *fax_receiver_entry;
+
+    GtkWidget *email_receiver_entry;
+    GtkWidget *email_subject_entry;
+    GtkWidget *email_text_widget;
+    GtkWidget *email_html_mode_widget;
 
     GdkPixmap *window_icon_pixmap;
     GdkBitmap *window_icon_mask;        
@@ -685,7 +735,6 @@ typedef struct Xsane
     Preview   *preview;
     int preview_gamma_size;
     int mode;
-    int medium_nr;
 
     int main_window_fixed;
     int mode_selection;
@@ -711,6 +760,9 @@ typedef struct Xsane
     GtkWidget *filename_entry;
     GtkWidget *filetype_option_menu;
 
+    /* for all modes */
+    GtkWidget *cms_function_option_menu;
+
     /* saving and transformation values: */
     FILE *out;
     int xsane_mode;
@@ -770,8 +822,6 @@ typedef struct Xsane
 
     int copy_number;
     double zoom;
-    double zoom_x;
-    double zoom_y;
     double resolution;
     double resolution_x;
     double resolution_y;
@@ -790,29 +840,32 @@ typedef struct Xsane
     GtkWidget *show_standard_options_widget;
     GtkWidget *show_advanced_options_widget;
     GtkWidget *show_resolution_list_widget;
-    GtkObject *zoom_widget;
-    GtkObject *gamma_widget;
-    GtkObject *gamma_red_widget;
-    GtkObject *gamma_green_widget;
-    GtkObject *gamma_blue_widget;
-    GtkObject *brightness_widget;
-    GtkObject *brightness_red_widget;
-    GtkObject *brightness_green_widget;
-    GtkObject *brightness_blue_widget;
-    GtkObject *contrast_widget;
-    GtkObject *contrast_red_widget;
-    GtkObject *contrast_green_widget;
-    GtkObject *contrast_blue_widget;
-    GtkObject *threshold_widget;
+    GtkWidget *enable_color_management_widget;
+    GtkWidget *edit_medium_definition_widget;
+    GtkWidget *zoom_widget;
+    GtkWidget *gamma_widget;
+    GtkWidget *gamma_red_widget;
+    GtkWidget *gamma_green_widget;
+    GtkWidget *gamma_blue_widget;
+    GtkWidget *brightness_widget;
+    GtkWidget *brightness_red_widget;
+    GtkWidget *brightness_green_widget;
+    GtkWidget *brightness_blue_widget;
+    GtkWidget *contrast_widget;
+    GtkWidget *contrast_red_widget;
+    GtkWidget *contrast_green_widget;
+    GtkWidget *contrast_blue_widget;
+    GtkWidget *threshold_widget;
 
-    SANE_Int xsane_colors;
+    SANE_Int xsane_channels;
     SANE_Bool scanner_gamma_color;
     SANE_Bool scanner_gamma_gray;
 
-    int mail_project_save;
-    int mail_html_mode;
+    int email_project_save;
+    int email_html_mode;
 
     GtkWidget *outputfilename_entry;
+    GtkWidget *adf_pages_max_entry;
     GtkWidget *copy_number_entry;
 
     gfloat *free_gamma_data, *free_gamma_data_red, *free_gamma_data_green, *free_gamma_data_blue;
@@ -825,13 +878,16 @@ typedef struct Xsane
     char *fax_filename;
     char *fax_receiver;
 
-    float mail_progress_val;
-    int   mail_progress_size;
-    int   mail_progress_bytes;
-    char *mail_status;
-    char *mail_filename;
-    char *mail_receiver;
-    char *mail_subject;
+    float email_progress_val;
+    int   email_progress_size;
+    int   email_progress_bytes;
+    char *email_status;
+    char *email_filename;
+    char *email_receiver;
+    char *email_subject;
+
+    char *multipage_status;
+    char *multipage_filename;
 
     int block_update_param;
     int block_enhancement_update;
@@ -849,14 +905,16 @@ typedef struct Xsane
    rc_io-routintes that are based on double, int, ... */
 
     /* window position and geometry */
-    int shell_posx;
-    int shell_posy;
-    int shell_height;
-    int shell_width;
-    int standard_options_shell_posx;
-    int standard_options_shell_posy;
-    int advanced_options_shell_posx;
-    int advanced_options_shell_posy;
+    int dialog_posx;
+    int dialog_posy;
+    int dialog_height;
+    int dialog_width;
+    int project_dialog_posx;
+    int project_dialog_posy;
+    int standard_options_dialog_posx;
+    int standard_options_dialog_posy;
+    int advanced_options_dialog_posx;
+    int advanced_options_dialog_posy;
     int histogram_dialog_posx;
     int histogram_dialog_posy;
     int gamma_dialog_posx;
@@ -913,10 +971,16 @@ typedef struct Xsane
     int enhancement_rgb_default;
     int negative;
     int show_preview;
+
+    int enable_color_management; 
+    char *scanner_active_icm_profile;
+    char *scanner_default_color_icm_profile;
+    char *scanner_default_gray_icm_profile;
+
     int print_filenames;
     int force_filename;
     char *external_filename;
-    char *adf_scansource;
+
 
 /* -------------------------------------------------- */
 
@@ -943,15 +1007,23 @@ typedef struct XsaneSetup
   GtkWidget *printer_gamma_red_entry;
   GtkWidget *printer_gamma_green_entry;
   GtkWidget *printer_gamma_blue_entry;
+  GtkWidget *printer_icm_profile_entry;
+  GtkWidget *printer_embed_csa_button;
+  GtkWidget *printer_embed_crd_button;
+  GtkWidget *printer_cms_bpc_button;
   GtkWidget *printer_width_entry;
   GtkWidget *printer_height_entry;
+  GtkWidget *printer_ps_flatedecoded_button;
 
   GtkWidget *jpeg_image_quality_scale;
-  GtkWidget *pnm_image_compression_scale;
+  GtkWidget *png_image_compression_scale;
+  GtkWidget *tiff_image_zip_compression_scale;
   GtkWidget *save_devprefs_at_exit_button;
   GtkWidget *overwrite_warning_button;
   GtkWidget *increase_filename_counter_button;
   GtkWidget *skip_existing_numbers_button;
+  GtkWidget *save_ps_flatedecoded_button;
+  GtkWidget *save_pdf_flatedecoded_button;
   GtkWidget *save_pnm16_as_ascii_button;
   GtkWidget *reduce_16bit_to_8bit_button;
 
@@ -985,18 +1057,19 @@ typedef struct XsaneSetup
   GtkWidget *fax_leftoffset_entry;
   GtkWidget *fax_bottomoffset_entry;
   GtkWidget *fax_height_entry;
+  GtkWidget *fax_ps_flatedecoded_button;
+
   GtkWidget *tmp_path_entry;
 
-  GtkWidget *mail_smtp_server_entry;
-  GtkWidget *mail_smtp_port_entry;
-  GtkWidget *mail_from_entry;
-  GtkWidget *mail_reply_to_entry;
-  GtkWidget *mail_pop3_authentification_entry;
-  GtkWidget *mail_pop3_server_entry;
-  GtkWidget *mail_pop3_port_entry;
-  GtkWidget *mail_pop3_user_entry;
-  GtkWidget *mail_pop3_pass_entry;
-  GtkWidget *mail_viewer_entry;
+  GtkWidget *email_smtp_server_entry;
+  GtkWidget *email_smtp_port_entry;
+  GtkWidget *email_from_entry;
+  GtkWidget *email_reply_to_entry;
+  GtkWidget *email_auth_user_entry;
+  GtkWidget *email_auth_pass_entry;
+  GtkWidget *email_pop3_server_entry;
+  GtkWidget *email_pop3_port_entry;
+  GtkWidget *pop3_vbox;
 
   GtkWidget *ocr_command_entry;
   GtkWidget *ocr_inputfile_option_entry;
@@ -1005,12 +1078,23 @@ typedef struct XsaneSetup
   GtkWidget *ocr_gui_outfd_option_entry;
   GtkWidget *ocr_progress_keyword_entry;
 
+  GtkWidget *cms_intent_option_menu;
+  GtkWidget *cms_bpc_button;
+  GtkWidget *embed_scanner_icm_profile_for_gimp_button;
+  GtkWidget *scanner_default_color_icm_profile_entry;
+  GtkWidget *scanner_default_gray_icm_profile_entry;
+  GtkWidget *display_icm_profile_entry;
+  GtkWidget *custom_proofing_icm_profile_entry;
+  GtkWidget *working_color_space_icm_profile_entry;
+
   int filename_counter_len;
 
   int tiff_compression16_nr;
   int tiff_compression8_nr;
   int tiff_compression1_nr;
 
+  int email_authentication;
+
   int show_range_mode;
   int lineart_mode;
 
